package java.time.format;

import java.time.temporal.WeekFields;
import java.util.concurrent.ConcurrentMap;
import java.text.ParsePosition;
import java.util.AbstractMap;
import java.time.zone.ZoneRulesProvider;
import java.time.Instant;
import java.util.concurrent.ConcurrentHashMap;
import java.util.Arrays;
import sun.util.locale.provider.TimeZoneNameUtility;
import java.util.HashSet;
import java.lang.ref.SoftReference;
import java.time.LocalDateTime;
import java.time.chrono.IsoChronology;
import java.time.temporal.ValueRange;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.math.BigInteger;
import java.time.DateTimeException;
import java.time.temporal.IsoFields;
import java.util.HashMap;
import java.time.ZoneOffset;
import java.time.temporal.TemporalAccessor;
import java.time.temporal.ChronoField;
import java.util.Set;
import java.time.temporal.TemporalQueries;
import java.util.Iterator;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.time.chrono.ChronoLocalDate;
import java.util.ArrayList;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.util.Objects;
import java.util.Locale;
import java.time.chrono.Chronology;
import java.util.Comparator;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.util.List;
import java.time.ZoneId;
import java.time.temporal.TemporalQuery;

public final class DateTimeFormatterBuilder
{
    private static final TemporalQuery<ZoneId> QUERY_REGION_ONLY;
    private DateTimeFormatterBuilder active;
    private final DateTimeFormatterBuilder parent;
    private final List<DateTimeFormatterBuilder.DateTimePrinterParser> printerParsers;
    private final boolean optional;
    private int padNextWidth;
    private char padNextChar;
    private int valueParserIndex;
    private static final Map<Character, TemporalField> FIELD_MAP;
    static final Comparator<String> LENGTH_SORT;
    
    public static String getLocalizedDateTimePattern(FormatStyle formatStyle, FormatStyle formatStyle2, Chronology obj, Locale obj2) {
        Objects.requireNonNull(obj2, "locale");
        Objects.requireNonNull(obj, "chrono");
        if (formatStyle == null && formatStyle2 == null) {
            throw new IllegalArgumentException("Either dateStyle or timeStyle must be non-null");
        }
        return LocaleProviderAdapter.getResourceBundleBased().getLocaleResources(obj2).getJavaTimeDateTimePattern(convertStyle(formatStyle2), convertStyle(formatStyle), obj.getCalendarType());
    }
    
    private static int convertStyle(FormatStyle formatStyle) {
        if (formatStyle == null) {
            return -1;
        }
        return formatStyle.ordinal();
    }
    
    public DateTimeFormatterBuilder() {
        this.active = this;
        this.printerParsers = new ArrayList();
        this.valueParserIndex = -1;
        this.parent = null;
        this.optional = false;
    }
    
    private DateTimeFormatterBuilder(DateTimeFormatterBuilder parent, boolean optional) {
        this.active = this;
        this.printerParsers = new ArrayList();
        this.valueParserIndex = -1;
        this.parent = parent;
        this.optional = optional;
    }
    
    public DateTimeFormatterBuilder parseCaseSensitive() {
        this.appendInternal(DateTimeFormatterBuilder.SettingsParser.SENSITIVE);
        return this;
    }
    
    public DateTimeFormatterBuilder parseCaseInsensitive() {
        this.appendInternal(DateTimeFormatterBuilder.SettingsParser.INSENSITIVE);
        return this;
    }
    
    public DateTimeFormatterBuilder parseStrict() {
        this.appendInternal(DateTimeFormatterBuilder.SettingsParser.STRICT);
        return this;
    }
    
    public DateTimeFormatterBuilder parseLenient() {
        this.appendInternal(DateTimeFormatterBuilder.SettingsParser.LENIENT);
        return this;
    }
    
    public DateTimeFormatterBuilder parseDefaulting(TemporalField obj, long n) {
        Objects.requireNonNull(obj, "field");
        this.appendInternal(new DateTimeFormatterBuilder.DefaultValueParser(obj, n));
        return this;
    }
    
    public DateTimeFormatterBuilder appendValue(TemporalField obj) {
        Objects.requireNonNull(obj, "field");
        this.appendValue(new DateTimeFormatterBuilder.NumberPrinterParser(obj, 1, 19, SignStyle.NORMAL));
        return this;
    }
    
    public DateTimeFormatterBuilder appendValue(TemporalField obj, int i) {
        Objects.requireNonNull(obj, "field");
        if (i < 1 || i > 19) {
            throw new IllegalArgumentException("The width must be from 1 to 19 inclusive but was " + i);
        }
        this.appendValue(new DateTimeFormatterBuilder.NumberPrinterParser(obj, i, i, SignStyle.NOT_NEGATIVE));
        return this;
    }
    
    public DateTimeFormatterBuilder appendValue(TemporalField obj, int n, int n2, SignStyle obj2) {
        if (n == n2 && obj2 == SignStyle.NOT_NEGATIVE) {
            return this.appendValue(obj, n2);
        }
        Objects.requireNonNull(obj, "field");
        Objects.requireNonNull(obj2, "signStyle");
        if (n < 1 || n > 19) {
            throw new IllegalArgumentException("The minimum width must be from 1 to 19 inclusive but was " + n);
        }
        if (n2 < 1 || n2 > 19) {
            throw new IllegalArgumentException("The maximum width must be from 1 to 19 inclusive but was " + n2);
        }
        if (n2 < n) {
            throw new IllegalArgumentException("The maximum width must exceed or equal the minimum width but " + n2 + " < " + n);
        }
        this.appendValue(new DateTimeFormatterBuilder.NumberPrinterParser(obj, n, n2, obj2));
        return this;
    }
    
    public DateTimeFormatterBuilder appendValueReduced(TemporalField obj, int n, int n2, int n3) {
        Objects.requireNonNull(obj, "field");
        this.appendValue(new DateTimeFormatterBuilder.ReducedPrinterParser(obj, n, n2, n3, null));
        return this;
    }
    
    public DateTimeFormatterBuilder appendValueReduced(TemporalField obj, int n, int n2, ChronoLocalDate obj2) {
        Objects.requireNonNull(obj, "field");
        Objects.requireNonNull(obj2, "baseDate");
        this.appendValue(new DateTimeFormatterBuilder.ReducedPrinterParser(obj, n, n2, 0, obj2));
        return this;
    }
    
    private DateTimeFormatterBuilder appendValue(DateTimeFormatterBuilder.NumberPrinterParser numberPrinterParser) {
        if (this.active.valueParserIndex >= 0) {
            int valueParserIndex = this.active.valueParserIndex;
            DateTimeFormatterBuilder.NumberPrinterParser numberPrinterParser2 = (DateTimeFormatterBuilder.NumberPrinterParser)this.active.printerParsers.get(valueParserIndex);
            DateTimeFormatterBuilder.NumberPrinterParser numberPrinterParser3;
            if (numberPrinterParser.minWidth == numberPrinterParser.maxWidth && numberPrinterParser.signStyle == SignStyle.NOT_NEGATIVE) {
                numberPrinterParser3 = numberPrinterParser2.withSubsequentWidth(numberPrinterParser.maxWidth);
                this.appendInternal(numberPrinterParser.withFixedWidth());
                this.active.valueParserIndex = valueParserIndex;
            }
            else {
                numberPrinterParser3 = numberPrinterParser2.withFixedWidth();
                this.active.valueParserIndex = this.appendInternal(numberPrinterParser);
            }
            this.active.printerParsers.set(valueParserIndex, numberPrinterParser3);
            return this;
        }
        else {
            this.active.valueParserIndex = this.appendInternal(numberPrinterParser);
            return this;
        }
    }
    
    public DateTimeFormatterBuilder appendFraction(TemporalField temporalField, int n, int n2, boolean b) {
        this.appendInternal(new DateTimeFormatterBuilder.FractionPrinterParser(temporalField, n, n2, b));
        return this;
    }
    
    public DateTimeFormatterBuilder appendText(TemporalField temporalField) {
        return this.appendText(temporalField, TextStyle.FULL);
    }
    
    public DateTimeFormatterBuilder appendText(TemporalField obj, TextStyle obj2) {
        Objects.requireNonNull(obj, "field");
        Objects.requireNonNull(obj2, "textStyle");
        this.appendInternal(new DateTimeFormatterBuilder.TextPrinterParser(obj, obj2, DateTimeTextProvider.getInstance()));
        return this;
    }
    
    public DateTimeFormatterBuilder appendText(TemporalField obj, Map<Long, String> map) {
        Objects.requireNonNull(obj, "field");
        Objects.requireNonNull(map, "textLookup");
        this.appendInternal(new DateTimeFormatterBuilder.TextPrinterParser(obj, TextStyle.FULL, new DateTimeTextProvider() {
            
            @Override
            public String getText(Chronology chronology, TemporalField temporalField, long n, TextStyle textStyle, Locale locale) {
                return store.getText(n, textStyle);
            }
            
            @Override
            public String getText(TemporalField temporalField, long n, TextStyle textStyle, Locale locale) {
                return store.getText(n, textStyle);
            }
            
            @Override
            public Iterator<Map.Entry<String, Long>> getTextIterator(Chronology chronology, TemporalField temporalField, TextStyle textStyle, Locale locale) {
                return store.getTextIterator(textStyle);
            }
            
            @Override
            public Iterator<Map.Entry<String, Long>> getTextIterator(TemporalField temporalField, TextStyle textStyle, Locale locale) {
                return store.getTextIterator(textStyle);
            }
        }));
        return this;
    }
    
    public DateTimeFormatterBuilder appendInstant() {
        this.appendInternal(new DateTimeFormatterBuilder.InstantPrinterParser(-2));
        return this;
    }
    
    public DateTimeFormatterBuilder appendInstant(int i) {
        if (i < -1 || i > 9) {
            throw new IllegalArgumentException("The fractional digits must be from -1 to 9 inclusive but was " + i);
        }
        this.appendInternal(new InstantPrinterParser(i));
        return this;
    }
    
    public DateTimeFormatterBuilder appendOffsetId() {
        this.appendInternal(DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_Z);
        return this;
    }
    
    public DateTimeFormatterBuilder appendOffset(String s, String s2) {
        this.appendInternal(new OffsetIdPrinterParser(s, s2));
        return this;
    }
    
    public DateTimeFormatterBuilder appendLocalizedOffset(TextStyle obj) {
        Objects.requireNonNull(obj, "style");
        if (obj != TextStyle.FULL && obj != TextStyle.SHORT) {
            throw new IllegalArgumentException("Style must be either full or short");
        }
        this.appendInternal(new LocalizedOffsetIdPrinterParser(obj));
        return this;
    }
    
    public DateTimeFormatterBuilder appendZoneId() {
        this.appendInternal(new DateTimeFormatterBuilder.ZoneIdPrinterParser(TemporalQueries.zoneId(), "ZoneId()"));
        return this;
    }
    
    public DateTimeFormatterBuilder appendZoneRegionId() {
        this.appendInternal(new DateTimeFormatterBuilder.ZoneIdPrinterParser(DateTimeFormatterBuilder.QUERY_REGION_ONLY, "ZoneRegionId()"));
        return this;
    }
    
    public DateTimeFormatterBuilder appendZoneOrOffsetId() {
        this.appendInternal(new DateTimeFormatterBuilder.ZoneIdPrinterParser(TemporalQueries.zone(), "ZoneOrOffsetId()"));
        return this;
    }
    
    public DateTimeFormatterBuilder appendZoneText(TextStyle textStyle) {
        this.appendInternal(new DateTimeFormatterBuilder.ZoneTextPrinterParser(textStyle, null));
        return this;
    }
    
    public DateTimeFormatterBuilder appendZoneText(TextStyle textStyle, Set<ZoneId> obj) {
        Objects.requireNonNull(obj, "preferredZones");
        this.appendInternal(new DateTimeFormatterBuilder.ZoneTextPrinterParser(textStyle, obj));
        return this;
    }
    
    public DateTimeFormatterBuilder appendChronologyId() {
        this.appendInternal(new DateTimeFormatterBuilder.ChronoPrinterParser(null));
        return this;
    }
    
    public DateTimeFormatterBuilder appendChronologyText(TextStyle obj) {
        Objects.requireNonNull(obj, "textStyle");
        this.appendInternal(new ChronoPrinterParser(obj));
        return this;
    }
    
    public DateTimeFormatterBuilder appendLocalized(FormatStyle formatStyle, FormatStyle formatStyle2) {
        if (formatStyle == null && formatStyle2 == null) {
            throw new IllegalArgumentException("Either the date or time style must be non-null");
        }
        this.appendInternal(new LocalizedPrinterParser(formatStyle, formatStyle2));
        return this;
    }
    
    public DateTimeFormatterBuilder appendLiteral(char c) {
        this.appendInternal(new CharLiteralPrinterParser(c));
        return this;
    }
    
    public DateTimeFormatterBuilder appendLiteral(String obj) {
        Objects.requireNonNull(obj, "literal");
        if (obj.length() > 0) {
            if (obj.length() == 1) {
                this.appendInternal(new DateTimeFormatterBuilder.CharLiteralPrinterParser(obj.charAt(0)));
            }
            else {
                this.appendInternal(new StringLiteralPrinterParser(obj));
            }
        }
        return this;
    }
    
    public DateTimeFormatterBuilder append(DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        this.appendInternal(obj.toPrinterParser(false));
        return this;
    }
    
    public DateTimeFormatterBuilder appendOptional(DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        this.appendInternal(obj.toPrinterParser(true));
        return this;
    }
    
    public DateTimeFormatterBuilder appendPattern(String obj) {
        Objects.requireNonNull(obj, "pattern");
        this.parsePattern(obj);
        return this;
    }
    
    private void parsePattern(String s) {
        for (int i = 0; i < s.length(); ++i) {
            int c = s.charAt(i);
            if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
                int n2;
                for (n2 = i++; i < s.length() && s.charAt(i) == c; ++i) {}
                int n3 = i - n2;
                if (c == 'p') {
                    int n4 = 0;
                    if (i < s.length()) {
                        c = s.charAt(i);
                        if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
                            n4 = n3;
                            int n5;
                            for (n5 = i++; i < s.length() && s.charAt(i) == c; ++i) {}
                            n3 = i - n5;
                        }
                    }
                    if (n4 == 0) {
                        throw new IllegalArgumentException("Pad letter 'p' must be followed by valid pad pattern: " + s);
                    }
                    this.padNext(n4);
                }
                TemporalField temporalField = (TemporalField)DateTimeFormatterBuilder.FIELD_MAP.get(Character.valueOf((char)c));
                if (temporalField != null) {
                    this.parseField((char)c, n3, temporalField);
                }
                else if (c == 'z') {
                    if (n3 > 4) {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                    if (n3 == 4) {
                        this.appendZoneText(TextStyle.FULL);
                    }
                    else {
                        this.appendZoneText(TextStyle.SHORT);
                    }
                }
                else if (c == 'V') {
                    if (n3 != 2) {
                        throw new IllegalArgumentException("Pattern letter count must be 2: " + (char)c);
                    }
                    this.appendZoneId();
                }
                else if (c == 'Z') {
                    if (n3 < 4) {
                        this.appendOffset("+HHMM", "+0000");
                    }
                    else if (n3 == 4) {
                        this.appendLocalizedOffset(TextStyle.FULL);
                    }
                    else if (n3 == 5) {
                        this.appendOffset("+HH:MM:ss", "Z");
                    }
                    else {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                }
                else if (c == 'O') {
                    if (n3 == 1) {
                        this.appendLocalizedOffset(TextStyle.SHORT);
                    }
                    else if (n3 == 4) {
                        this.appendLocalizedOffset(TextStyle.FULL);
                    }
                    else {
                        throw new IllegalArgumentException("Pattern letter count must be 1 or 4: " + (char)c);
                    }
                }
                else if (c == 'X') {
                    if (n3 > 5) {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                    this.appendOffset(DateTimeFormatterBuilder.OffsetIdPrinterParser.PATTERNS[n3 + (((n3 == 1) ? false : true) ? 1 : 0)], "Z");
                }
                else if (c == 'x') {
                    if (n3 > 5) {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                    this.appendOffset(DateTimeFormatterBuilder.OffsetIdPrinterParser.PATTERNS[n3 + (((n3 == 1) ? false : true) ? 1 : 0)], (n3 == 1) ? "+00" : ((n3 % 2 == 0) ? "+0000" : "+00:00"));
                }
                else if (c == 'W') {
                    if (n3 > 1) {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                    this.appendInternal(new DateTimeFormatterBuilder.WeekBasedFieldPrinterParser((char)c, n3));
                }
                else if (c == 'w') {
                    if (n3 > 2) {
                        throw new IllegalArgumentException("Too many pattern letters: " + (char)c);
                    }
                    this.appendInternal(new DateTimeFormatterBuilder.WeekBasedFieldPrinterParser((char)c, n3));
                }
                else if (c == 'Y') {
                    this.appendInternal(new DateTimeFormatterBuilder.WeekBasedFieldPrinterParser((char)c, n3));
                }
                else {
                    throw new IllegalArgumentException("Unknown pattern letter: " + (char)c);
                }
                --i;
            }
            else if (c == '\'') {
                int n;
                for (n = i++; i < s.length(); ++i) {
                    if (s.charAt(i) == '\'') {
                        if (i + 1 < s.length() && s.charAt(i + 1) == '\'') {
                            ++i;
                        }
                        else {
                            break;
                        }
                    }
                }
                if (i >= s.length()) {
                    throw new IllegalArgumentException("Pattern ends with an incomplete string literal: " + s);
                }
                String substring = s.substring(n + 1, i);
                if (substring.length() == 0) {
                    this.appendLiteral('\'');
                }
                else {
                    this.appendLiteral(substring.replace("''", "'"));
                }
            }
            else if (c == '[') {
                this.optionalStart();
            }
            else if (c == ']') {
                if (this.active.parent == null) {
                    throw new IllegalArgumentException("Pattern invalid as it contains ] without previous [");
                }
                this.optionalEnd();
            }
            else {
                if (c == '{' || c == '}' || c == '#') {
                    throw new IllegalArgumentException("Pattern includes reserved character: '" + (char)c + "'");
                }
                this.appendLiteral((char)c);
            }
        }
    }
    
    private void parseField(char c, int n, TemporalField temporalField) {
        boolean n2 = false;
        switch (c) {
            case 'u':
            case 'y': {
                if (n == 2) {
                    this.appendValueReduced(temporalField, 2, 2, DateTimeFormatterBuilder.ReducedPrinterParser.BASE_DATE);
                    break;
                }
                if (n < 4) {
                    this.appendValue(temporalField, n, 19, SignStyle.NORMAL);
                    break;
                }
                this.appendValue(temporalField, n, 19, SignStyle.EXCEEDS_PAD);
                break;
            }
            case 'c': {
                if (n == 2) {
                    throw new IllegalArgumentException("Invalid pattern \"cc\"");
                }
            }
            case 'L':
            case 'q': {
                n2 = true;
                break;
            }
            case 'E':
            case 'M':
            case 'Q':
            case 'e': {
                Label_0308: {
                    switch (n) {
                        case 1:
                        case 2: {
                            if (c == 'c' || c == 'e') {
                                this.appendInternal(new DateTimeFormatterBuilder.WeekBasedFieldPrinterParser(c, n));
                                break;
                            }
                            if (c == 'E') {
                                this.appendText(temporalField, TextStyle.SHORT);
                                break;
                            }
                            if (n == 1) {
                                this.appendValue(temporalField);
                                break;
                            }
                            this.appendValue(temporalField, 2);
                            break;
                        }
                        case 3: {
                            this.appendText(temporalField, n2 ? TextStyle.SHORT_STANDALONE : TextStyle.SHORT);
                            break;
                        }
                        case 4: {
                            this.appendText(temporalField, n2 ? TextStyle.FULL_STANDALONE : TextStyle.FULL);
                            break;
                        }
                        case 5: {
                            this.appendText(temporalField, n2 ? TextStyle.NARROW_STANDALONE : TextStyle.NARROW);
                            break;
                        }
                        default: {
                            throw new IllegalArgumentException("Too many pattern letters: " + c);
                        }
                    }
                }
                break;
            }
            case 'a': {
                if (n == 1) {
                    this.appendText(temporalField, TextStyle.SHORT);
                    break;
                }
                throw new IllegalArgumentException("Too many pattern letters: " + c);
            }
            case 'G': {
                switch (n) {
                    case 1:
                    case 2:
                    case 3: {
                        this.appendText(temporalField, TextStyle.SHORT);
                        break;
                    }
                    case 4: {
                        this.appendText(temporalField, TextStyle.FULL);
                        break;
                    }
                    case 5: {
                        this.appendText(temporalField, TextStyle.NARROW);
                        break;
                    }
                    default: {
                        throw new IllegalArgumentException("Too many pattern letters: " + c);
                    }
                }
                break;
            }
            case 'S': {
                this.appendFraction(ChronoField.NANO_OF_SECOND, n, n, false);
                break;
            }
            case 'F': {
                if (n == 1) {
                    this.appendValue(temporalField);
                    break;
                }
                throw new IllegalArgumentException("Too many pattern letters: " + c);
            }
            case 'H':
            case 'K':
            case 'd':
            case 'h':
            case 'k':
            case 'm':
            case 's': {
                if (n == 1) {
                    this.appendValue(temporalField);
                    break;
                }
                if (n == 2) {
                    this.appendValue(temporalField, n);
                    break;
                }
                throw new IllegalArgumentException("Too many pattern letters: " + c);
            }
            case 'D': {
                if (n == 1) {
                    this.appendValue(temporalField);
                    break;
                }
                if (n <= 3) {
                    this.appendValue(temporalField, n);
                    break;
                }
                throw new IllegalArgumentException("Too many pattern letters: " + c);
            }
            default: {
                if (n == 1) {
                    this.appendValue(temporalField);
                    break;
                }
                this.appendValue(temporalField, n);
                break;
            }
        }
    }
    
    public DateTimeFormatterBuilder padNext(int n) {
        return this.padNext(n, ' ');
    }
    
    public DateTimeFormatterBuilder padNext(int n, char padNextChar) {
        if (n < 1) {
            throw new IllegalArgumentException("The pad width must be at least one but was " + n);
        }
        this.active.padNextWidth = n;
        this.active.padNextChar = padNextChar;
        this.active.valueParserIndex = -1;
        return this;
    }
    
    public DateTimeFormatterBuilder optionalStart() {
        this.active.valueParserIndex = -1;
        this.active = new DateTimeFormatterBuilder(this.active, true);
        return this;
    }
    
    public DateTimeFormatterBuilder optionalEnd() {
        if (this.active.parent == null) {
            throw new IllegalStateException("Cannot call optionalEnd() as there was no previous call to optionalStart()");
        }
        if (this.active.printerParsers.size() > 0) {
            DateTimeFormatterBuilder.CompositePrinterParser compositePrinterParser = new DateTimeFormatterBuilder.CompositePrinterParser(this.active.printerParsers, this.active.optional);
            this.active = this.active.parent;
            this.appendInternal(compositePrinterParser);
            return this;
        }
        this.active = this.active.parent;
        return this;
    }
    
    private int appendInternal(DateTimeFormatterBuilder.DateTimePrinterParser obj) {
        Objects.requireNonNull(obj, "pp");
        if (this.active.padNextWidth > 0) {
            if (obj != null) {
                obj = new DateTimeFormatterBuilder.PadPrinterParserDecorator(obj, this.active.padNextWidth, this.active.padNextChar);
            }
            this.active.padNextWidth = 0;
            this.active.padNextChar = '\0';
        }
        this.active.printerParsers.add(obj);
        this.active.valueParserIndex = -1;
        return this.active.printerParsers.size() - 1;
    }
    
    public DateTimeFormatter toFormatter() {
        return this.toFormatter(Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public DateTimeFormatter toFormatter(Locale locale) {
        return this.toFormatter(locale, ResolverStyle.SMART, null);
    }
    
    DateTimeFormatter toFormatter(ResolverStyle resolverStyle, Chronology chronology) {
        return this.toFormatter(Locale.getDefault(Locale.Category.FORMAT), resolverStyle, chronology);
    }
    
    private DateTimeFormatter toFormatter(Locale obj, ResolverStyle resolverStyle, Chronology chronology) {
        Objects.requireNonNull(obj, "locale");
        while (this.active.parent != null) {
            this.optionalEnd();
        }
        return new DateTimeFormatter(new DateTimeFormatterBuilder.CompositePrinterParser(this.printerParsers, false), obj, DecimalStyle.STANDARD, resolverStyle, null, chronology, null);
    }
    
    static {
        QUERY_REGION_ONLY = (temporalAccessor/* java.time.temporal.TemporalAccessor, */ -> {
            ZoneId zoneId = (ZoneId)temporalAccessor.query(TemporalQueries.zoneId());
            return (zoneId != null && !(zoneId instanceof ZoneOffset)) ? zoneId : null;
        });
        (FIELD_MAP = new HashMap()).put(Character.valueOf('G'), ChronoField.ERA);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('y'), ChronoField.YEAR_OF_ERA);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('u'), ChronoField.YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('Q'), IsoFields.QUARTER_OF_YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('q'), IsoFields.QUARTER_OF_YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('M'), ChronoField.MONTH_OF_YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('L'), ChronoField.MONTH_OF_YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('D'), ChronoField.DAY_OF_YEAR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('d'), ChronoField.DAY_OF_MONTH);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('F'), ChronoField.ALIGNED_DAY_OF_WEEK_IN_MONTH);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('E'), ChronoField.DAY_OF_WEEK);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('c'), ChronoField.DAY_OF_WEEK);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('e'), ChronoField.DAY_OF_WEEK);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('a'), ChronoField.AMPM_OF_DAY);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('H'), ChronoField.HOUR_OF_DAY);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('k'), ChronoField.CLOCK_HOUR_OF_DAY);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('K'), ChronoField.HOUR_OF_AMPM);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('h'), ChronoField.CLOCK_HOUR_OF_AMPM);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('m'), ChronoField.MINUTE_OF_HOUR);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('s'), ChronoField.SECOND_OF_MINUTE);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('S'), ChronoField.NANO_OF_SECOND);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('A'), ChronoField.MILLI_OF_DAY);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('n'), ChronoField.NANO_OF_SECOND);
        DateTimeFormatterBuilder.FIELD_MAP.put(Character.valueOf('N'), ChronoField.NANO_OF_DAY);
        LENGTH_SORT = new Comparator<String>() {
            @Override
            public int compare(String s, String anotherString) {
                return (s.length() == anotherString.length()) ? s.compareTo(anotherString) : (s.length() - anotherString.length());
            }
        };
    }
    
    static final class CompositePrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers;
        private final boolean optional;
        
        CompositePrinterParser(List<DateTimeFormatterBuilder.DateTimePrinterParser> list, boolean b) {
            this((DateTimeFormatterBuilder.DateTimePrinterParser[])list.toArray(new DateTimeFormatterBuilder.DateTimePrinterParser[list.size()]), b);
        }
        
        CompositePrinterParser(DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers, boolean optional) {
            this.printerParsers = printerParsers;
            this.optional = optional;
        }
        
        public DateTimeFormatterBuilder.CompositePrinterParser withOptional(boolean b) {
            if (b == this.optional) {
                return this;
            }
            return new DateTimeFormatterBuilder.CompositePrinterParser(this.printerParsers, b);
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            int length = sb.length();
            if (this.optional) {
                dateTimePrintContext.startOptional();
            }
            try {
                DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers = this.printerParsers;
                int length2 = printerParsers.length;
                int i = 0;
                while (i < length2) {
                    if (!printerParsers[i].format(dateTimePrintContext, sb)) {
                        sb.setLength(length);
                        return true;
                    }
                    ++i;
                }
            }
            finally {
                if (this.optional) {
                    dateTimePrintContext.endOptional();
                }
            }
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            if (this.optional) {
                dateTimeParseContext.startOptional();
                DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers2 = this.printerParsers;
                int length2 = printerParsers2.length;
                int j = 0;
                while (j < length2) {
                    n = printerParsers2[j].parse(dateTimeParseContext, charSequence, n);
                    if (n < 0) {
                        dateTimeParseContext.endOptional(false);
                        return n;
                    }
                    ++j;
                }
                dateTimeParseContext.endOptional(true);
                return n;
            }
            DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers = this.printerParsers;
            int length = printerParsers.length;
            int i = 0;
            while (i < length) {
                n = printerParsers[i].parse(dateTimeParseContext, charSequence, n);
                if (n < 0) {
                    break;
                }
                ++i;
            }
            return n;
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            if (this.printerParsers != null) {
                sb.append(this.optional ? "[" : "(");
                DateTimeFormatterBuilder.DateTimePrinterParser[] printerParsers = this.printerParsers;
                for (int length = printerParsers.length, i = 0; i < length; ++i) {
                    sb.append(printerParsers[i]);
                }
                sb.append(this.optional ? "]" : ")");
            }
            return sb.toString();
        }
    }
    
    static final class PadPrinterParserDecorator implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final DateTimeFormatterBuilder.DateTimePrinterParser printerParser;
        private final int padWidth;
        private final char padChar;
        
        PadPrinterParserDecorator(DateTimeFormatterBuilder.DateTimePrinterParser printerParser, int padWidth, char padChar) {
            this.printerParser = printerParser;
            this.padWidth = padWidth;
            this.padChar = padChar;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            int length = sb.length();
            if (!this.printerParser.format(dateTimePrintContext, sb)) {
                return false;
            }
            int i = sb.length() - length;
            if (i > this.padWidth) {
                throw new DateTimeException("Cannot print as output of " + i + " characters exceeds pad width of " + this.padWidth);
            }
            for (int j = 0; j < this.padWidth - i; ++j) {
                sb.insert(length, this.padChar);
            }
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence subSequence, int n) {
            int strict = dateTimeParseContext.isStrict();
            if (n > subSequence.length()) {
                throw new IndexOutOfBoundsException();
            }
            if (n == subSequence.length()) {
                return ~n;
            }
            int length = n + this.padWidth;
            if (length > subSequence.length()) {
                if (!(strict == 0)) {
                    return ~n;
                }
                length = subSequence.length();
            }
            for (; n < length && dateTimeParseContext.charEquals(subSequence.charAt(n), this.padChar); ++n) {}
            subSequence = subSequence.subSequence(0, length);
            int parse = this.printerParser.parse(dateTimeParseContext, subSequence, n);
            if (parse != length && strict) {
                return ~(n + n);
            }
            return parse;
        }
        
        @Override
        public String toString() {
            return "Pad(" + this.printerParser + "," + this.padWidth + ((this.padChar == ' ') ? ")" : (",'" + this.padChar + "')"));
        }
    }
    
    enum SettingsParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        SENSITIVE, 
        INSENSITIVE, 
        STRICT, 
        LENIENT;
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            switch (this.ordinal()) {
                case 0: {
                    dateTimeParseContext.setCaseSensitive(true);
                    break;
                }
                case 1: {
                    dateTimeParseContext.setCaseSensitive(false);
                    break;
                }
                case 2: {
                    dateTimeParseContext.setStrict(true);
                    break;
                }
                case 3: {
                    dateTimeParseContext.setStrict(false);
                    break;
                }
            }
            return n;
        }
        
        @Override
        public String toString() {
            switch (this.ordinal()) {
                case 0: {
                    return "ParseCaseSensitive(true)";
                }
                case 1: {
                    return "ParseCaseSensitive(false)";
                }
                case 2: {
                    return "ParseStrict(true)";
                }
                case 3: {
                    return "ParseStrict(false)";
                }
                default: {
                    throw new IllegalStateException("Unreachable");
                }
            }
        }
    }
    
    static class DefaultValueParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TemporalField field;
        private final long value;
        
        DefaultValueParser(TemporalField field, long value) {
            this.field = field;
            this.value = value;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            if (dateTimeParseContext.getParsed(this.field) == null) {
                dateTimeParseContext.setParsedField(this.field, this.value, n, n);
            }
            return n;
        }
    }
    
    static final class CharLiteralPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final char literal;
        
        CharLiteralPrinterParser(char literal) {
            this.literal = literal;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            sb.append(this.literal);
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            if (n == charSequence.length()) {
                return ~n;
            }
            int char1 = charSequence.charAt(n);
            if (char1 != this.literal && (dateTimeParseContext.isCaseSensitive() || (Character.toUpperCase((char)char1) != Character.toUpperCase(this.literal) && Character.toLowerCase((char)char1) != Character.toLowerCase(this.literal)))) {
                return ~n;
            }
            return n + 1;
        }
        
        @Override
        public String toString() {
            if (this.literal == '\'') {
                return "''";
            }
            return "'" + this.literal + "'";
        }
    }
    
    static final class StringLiteralPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final String literal;
        
        StringLiteralPrinterParser(String literal) {
            this.literal = literal;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            sb.append(this.literal);
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            if (n > charSequence.length() || n < 0) {
                throw new IndexOutOfBoundsException();
            }
            if (!dateTimeParseContext.subSequenceEquals(charSequence, n, this.literal, 0, this.literal.length())) {
                return ~n;
            }
            return n + this.literal.length();
        }
        
        @Override
        public String toString() {
            return "'" + this.literal.replace("'", "''") + "'";
        }
    }
    
    static class NumberPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        static final long[] EXCEED_POINTS;
        final TemporalField field;
        final int minWidth;
        final int maxWidth;
        private final SignStyle signStyle;
        final int subsequentWidth;
        
        NumberPrinterParser(TemporalField field, int minWidth, int maxWidth, SignStyle signStyle) {
            this.field = field;
            this.minWidth = minWidth;
            this.maxWidth = maxWidth;
            this.signStyle = signStyle;
            this.subsequentWidth = 0;
        }
        
        protected NumberPrinterParser(TemporalField field, int minWidth, int maxWidth, SignStyle signStyle, int subsequentWidth) {
            this.field = field;
            this.minWidth = minWidth;
            this.maxWidth = maxWidth;
            this.signStyle = signStyle;
            this.subsequentWidth = subsequentWidth;
        }
        
        DateTimeFormatterBuilder.NumberPrinterParser withFixedWidth() {
            if (this.subsequentWidth == -1) {
                return this;
            }
            return new DateTimeFormatterBuilder.NumberPrinterParser(this.field, this.minWidth, this.maxWidth, this.signStyle, -1);
        }
        
        DateTimeFormatterBuilder.NumberPrinterParser withSubsequentWidth(int n) {
            return new DateTimeFormatterBuilder.NumberPrinterParser(this.field, this.minWidth, this.maxWidth, this.signStyle, this.subsequentWidth + n);
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(this.field);
            if (value == null) {
                return false;
            }
            long value2 = this.getValue(dateTimePrintContext, (long)value);
            DecimalStyle decimalStyle = dateTimePrintContext.getDecimalStyle();
            String s = (value2 == Long.MIN_VALUE) ? "9223372036854775808" : Long.toString(Math.abs(value2));
            if (s.length() > this.maxWidth) {
                throw new DateTimeException("Field " + this.field + " cannot be printed as the value " + value2 + " exceeds the maximum print width of " + this.maxWidth);
            }
            String convertNumberToI18N = decimalStyle.convertNumberToI18N(s);
            if (value2 >= 0L) {
                switch (this.signStyle) {
                    case EXCEEDS_PAD: {
                        if (this.minWidth < 19 && value2 >= DateTimeFormatterBuilder.NumberPrinterParser.EXCEED_POINTS[this.minWidth]) {
                            sb.append(decimalStyle.getPositiveSign());
                            break;
                        }
                        break;
                    }
                    case ALWAYS: {
                        sb.append(decimalStyle.getPositiveSign());
                        break;
                    }
                }
            }
            else {
                switch (this.signStyle) {
                    case EXCEEDS_PAD:
                    case ALWAYS:
                    case NORMAL: {
                        sb.append(decimalStyle.getNegativeSign());
                        break;
                    }
                    case NOT_NEGATIVE: {
                        throw new DateTimeException("Field " + this.field + " cannot be printed as the value " + value2 + " cannot be negative according to the SignStyle");
                    }
                }
            }
            for (int i = 0; i < this.minWidth - convertNumberToI18N.length(); ++i) {
                sb.append(decimalStyle.getZeroDigit());
            }
            sb.append(convertNumberToI18N);
            return true;
        }
        
        long getValue(DateTimePrintContext dateTimePrintContext, long n) {
            return n;
        }
        
        boolean isFixedWidth(DateTimeParseContext dateTimeParseContext) {
            return this.subsequentWidth == -1 || (this.subsequentWidth > 0 && this.minWidth == this.maxWidth && this.signStyle == SignStyle.NOT_NEGATIVE);
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int i) {
            int length = charSequence.length();
            if (i == length) {
                return ~i;
            }
            int char1 = charSequence.charAt(i);
            boolean n = false;
            boolean n2 = false;
            if (char1 == dateTimeParseContext.getDecimalStyle().getPositiveSign()) {
                if (!this.signStyle.parse(true, dateTimeParseContext.isStrict(), this.minWidth == this.maxWidth)) {
                    return ~i;
                }
                n2 = true;
                ++i;
            }
            else if (char1 == dateTimeParseContext.getDecimalStyle().getNegativeSign()) {
                if (!this.signStyle.parse(false, dateTimeParseContext.isStrict(), this.minWidth == this.maxWidth)) {
                    return ~i;
                }
                n = true;
                ++i;
            }
            else if (this.signStyle == SignStyle.ALWAYS && dateTimeParseContext.isStrict()) {
                return ~i;
            }
            int a = (dateTimeParseContext.isStrict() || this.isFixedWidth(dateTimeParseContext)) ? this.minWidth : 1;
            int n3 = i + a;
            if (n3 > length) {
                return ~i;
            }
            int max = ((dateTimeParseContext.isStrict() || this.isFixedWidth(dateTimeParseContext)) ? this.maxWidth : 9) + Math.max(this.subsequentWidth, 0);
            long val = 0L;
            BigInteger bigInteger = null;
            int j = 0;
            while (j < 2) {
                while (i < Math.min(i + max, length)) {
                    int convertToDigit = dateTimeParseContext.getDecimalStyle().convertToDigit(charSequence.charAt(i++));
                    if (convertToDigit < 0) {
                        if (--i < n3) {
                            return ~i;
                        }
                        break;
                    }
                    else if (i - i > 18) {
                        if (bigInteger == null) {
                            bigInteger = BigInteger.valueOf(val);
                        }
                        bigInteger = bigInteger.multiply(BigInteger.TEN).add(BigInteger.valueOf((long)convertToDigit));
                    }
                    else {
                        val = val * 10L + (long)convertToDigit;
                    }
                }
                if (this.subsequentWidth > 0 && j == 0) {
                    max = Math.max(a, i - i - this.subsequentWidth);
                    val = 0L;
                    bigInteger = null;
                    ++j;
                }
                else {
                    break;
                }
            }
            if (!(n == 0)) {
                if (bigInteger != null) {
                    if (bigInteger.equals(BigInteger.ZERO) && dateTimeParseContext.isStrict()) {
                        return ~(i - 1);
                    }
                    bigInteger = bigInteger.negate();
                }
                else {
                    if (val == 0L && dateTimeParseContext.isStrict()) {
                        return ~(i - 1);
                    }
                    val = -val;
                }
            }
            else if (this.signStyle == SignStyle.EXCEEDS_PAD && dateTimeParseContext.isStrict()) {
                int n4 = i - i;
                if (!(n2 == 0)) {
                    if (n4 <= this.minWidth) {
                        return ~(i - 1);
                    }
                }
                else if (n4 > this.minWidth) {
                    return ~i;
                }
            }
            if (bigInteger != null) {
                if (bigInteger.bitLength() > 63) {
                    bigInteger = bigInteger.divide(BigInteger.TEN);
                    --i;
                }
                return this.setValue(dateTimeParseContext, bigInteger.longValue(), i, i);
            }
            else {
                return this.setValue(dateTimeParseContext, val, i, i);
            }
        }
        
        int setValue(DateTimeParseContext dateTimeParseContext, long n, int n2, int n3) {
            return dateTimeParseContext.setParsedField(this.field, n, n2, n3);
        }
        
        @Override
        public String toString() {
            if (this.minWidth == 1 && this.maxWidth == 19 && this.signStyle == SignStyle.NORMAL) {
                return "Value(" + this.field + ")";
            }
            if (this.minWidth == this.maxWidth && this.signStyle == SignStyle.NOT_NEGATIVE) {
                return "Value(" + this.field + "," + this.minWidth + ")";
            }
            return "Value(" + this.field + "," + this.minWidth + "," + this.maxWidth + "," + this.signStyle + ")";
        }
        
        static {
            EXCEED_POINTS = new long[] { 0L, 10L, 100L, 1000L, 10000L, 100000L, 1000000L, 10000000L, 100000000L, 1000000000L, 10000000000L };
        }
    }
    
    static final class ReducedPrinterParser extends DateTimeFormatterBuilder.NumberPrinterParser
    {
        static final LocalDate BASE_DATE;
        private final int baseValue;
        private final ChronoLocalDate baseDate;
        
        ReducedPrinterParser(TemporalField temporalField, int i, int j, int n, ChronoLocalDate chronoLocalDate) {
            this(temporalField, i, j, n, chronoLocalDate, 0);
            if (i < 1 || i > 10) {
                throw new IllegalArgumentException("The minWidth must be from 1 to 10 inclusive but was " + i);
            }
            if (j < 1 || j > 10) {
                throw new IllegalArgumentException("The maxWidth must be from 1 to 10 inclusive but was " + i);
            }
            if (j < i) {
                throw new IllegalArgumentException("Maximum width must exceed or equal the minimum width but " + j + " < " + i);
            }
            if (chronoLocalDate == null) {
                if (!temporalField.range().isValidValue((long)n)) {
                    throw new IllegalArgumentException("The base value must be within the range of the field");
                }
                if ((long)n + DateTimeFormatterBuilder.ReducedPrinterParser.EXCEED_POINTS[j] > 2147483647L) {
                    throw new DateTimeException("Unable to add printer-parser as the range exceeds the capacity of an int");
                }
            }
            return;
        }
        
        private ReducedPrinterParser(TemporalField temporalField, int n, int n2, int baseValue, ChronoLocalDate baseDate, int n3) {
            super(temporalField, n, n2, SignStyle.NOT_NEGATIVE, n3);
            this.baseValue = baseValue;
            this.baseDate = baseDate;
        }
        
        @Override
        long getValue(DateTimePrintContext dateTimePrintContext, long a) {
            long abs = Math.abs(a);
            int n = this.baseValue;
            if (this.baseDate != null) {
                n = Chronology.from(dateTimePrintContext.getTemporal()).date(this.baseDate).get(this.field);
            }
            if (a >= (long)n && a < (long)n + DateTimeFormatterBuilder.ReducedPrinterParser.EXCEED_POINTS[this.minWidth]) {
                return abs % DateTimeFormatterBuilder.ReducedPrinterParser.EXCEED_POINTS[this.minWidth];
            }
            return abs % DateTimeFormatterBuilder.ReducedPrinterParser.EXCEED_POINTS[this.maxWidth];
        }
        
        @Override
        int setValue(DateTimeParseContext dateTimeParseContext, long n, int n2, int n3) {
            int n4 = this.baseValue;
            if (this.baseDate != null) {
                n4 = dateTimeParseContext.getEffectiveChronology().date(this.baseDate).get(this.field);
                dateTimeParseContext.addChronoChangedListener(p4/* java.time.chrono.Chronology, */ -> this.setValue(p0, p1, p3, n7));
            }
            if (n3 - n2 == this.minWidth && n >= 0L) {
                long n5 = DateTimeFormatterBuilder.ReducedPrinterParser.EXCEED_POINTS[this.minWidth];
                long n6 = (long)n4 - (long)n4 % n5;
                if (n4 > 0) {
                    n += n6;
                }
                else {
                    n = n6 - n;
                }
                if (n < (long)n4) {
                    n += n5;
                }
            }
            return dateTimeParseContext.setParsedField(this.field, n, n2, n3);
        }
        
        @Override
        DateTimeFormatterBuilder.ReducedPrinterParser withFixedWidth() {
            if (this.subsequentWidth == -1) {
                return this;
            }
            return new DateTimeFormatterBuilder.ReducedPrinterParser(this.field, this.minWidth, this.maxWidth, this.baseValue, this.baseDate, -1);
        }
        
        @Override
        DateTimeFormatterBuilder.ReducedPrinterParser withSubsequentWidth(int n) {
            return new DateTimeFormatterBuilder.ReducedPrinterParser(this.field, this.minWidth, this.maxWidth, this.baseValue, this.baseDate, this.subsequentWidth + n);
        }
        
        @Override
        boolean isFixedWidth(DateTimeParseContext dateTimeParseContext) {
            return dateTimeParseContext.isStrict() && super.isFixedWidth(dateTimeParseContext);
        }
        
        @Override
        public String toString() {
            return "ReducedValue(" + this.field + "," + this.minWidth + "," + this.maxWidth + "," + ((this.baseDate != null) ? this.baseDate : Integer.valueOf(this.baseValue)) + ")";
        }
        
        static {
            BASE_DATE = LocalDate.of(2000, 1, 1);
        }
    }
    
    static final class FractionPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TemporalField field;
        private final int minWidth;
        private final int maxWidth;
        private final boolean decimalPoint;
        
        FractionPrinterParser(TemporalField obj, int i, int j, boolean decimalPoint) {
            Objects.requireNonNull(obj, "field");
            if (!obj.range().isFixed()) {
                throw new IllegalArgumentException("Field must have a fixed set of values: " + obj);
            }
            if (i < 0 || i > 9) {
                throw new IllegalArgumentException("Minimum width must be from 0 to 9 inclusive but was " + i);
            }
            if (j < 1 || j > 9) {
                throw new IllegalArgumentException("Maximum width must be from 1 to 9 inclusive but was " + j);
            }
            if (j < i) {
                throw new IllegalArgumentException("Maximum width must exceed or equal the minimum width but " + j + " < " + i);
            }
            this.field = obj;
            this.minWidth = i;
            this.maxWidth = j;
            this.decimalPoint = decimalPoint;
            return;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(this.field);
            if (value == null) {
                return false;
            }
            DecimalStyle decimalStyle = dateTimePrintContext.getDecimalStyle();
            BigDecimal convertToFraction = this.convertToFraction((long)value);
            if (convertToFraction.scale() == 0) {
                if (this.minWidth > 0) {
                    if (this.decimalPoint) {
                        sb.append(decimalStyle.getDecimalSeparator());
                    }
                    for (int i = 0; i < this.minWidth; ++i) {
                        sb.append(decimalStyle.getZeroDigit());
                    }
                }
            }
            else {
                String convertNumberToI18N = decimalStyle.convertNumberToI18N(convertToFraction.setScale(Math.min(Math.max(convertToFraction.scale(), this.minWidth), this.maxWidth), RoundingMode.FLOOR).toPlainString().substring(2));
                if (this.decimalPoint) {
                    sb.append(decimalStyle.getDecimalSeparator());
                }
                sb.append(convertNumberToI18N);
            }
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int i) {
            int n = dateTimeParseContext.isStrict() ? this.minWidth : 0;
            int n2 = dateTimeParseContext.isStrict() ? this.maxWidth : 9;
            int length = charSequence.length();
            if (i == length) {
                return (n > 0) ? (~i) : i;
            }
            if (this.decimalPoint) {
                if (charSequence.charAt(i) != dateTimeParseContext.getDecimalStyle().getDecimalSeparator()) {
                    return (n > 0) ? (~i) : i;
                }
                ++i;
            }
            int n3 = i + n;
            if (n3 > length) {
                return ~i;
            }
            int min = Math.min(i + n2, length);
            int val = 0;
            while (i < min) {
                int convertToDigit = dateTimeParseContext.getDecimalStyle().convertToDigit(charSequence.charAt(i++));
                if (convertToDigit < 0) {
                    if (i < n3) {
                        return ~i;
                    }
                    --i;
                    break;
                }
                else {
                    val = val * 10 + convertToDigit;
                }
            }
            return dateTimeParseContext.setParsedField(this.field, this.convertFromFraction(new BigDecimal(val).movePointLeft(i - i)), i, i);
        }
        
        private BigDecimal convertToFraction(long n) {
            ValueRange range = this.field.range();
            range.checkValidValue(n, this.field);
            BigDecimal value = BigDecimal.valueOf(range.getMinimum());
            BigDecimal divide = BigDecimal.valueOf(n).subtract(value).divide(BigDecimal.valueOf(range.getMaximum()).subtract(value).add(BigDecimal.ONE), 9, RoundingMode.FLOOR);
            return (divide.compareTo(BigDecimal.ZERO) == 0) ? BigDecimal.ZERO : divide.stripTrailingZeros();
        }
        
        private long convertFromFraction(BigDecimal bigDecimal) {
            ValueRange range = this.field.range();
            BigDecimal value = BigDecimal.valueOf(range.getMinimum());
            return bigDecimal.multiply(BigDecimal.valueOf(range.getMaximum()).subtract(value).add(BigDecimal.ONE)).setScale(0, RoundingMode.FLOOR).add(value).longValueExact();
        }
        
        @Override
        public String toString() {
            return "Fraction(" + this.field + "," + this.minWidth + "," + this.maxWidth + (this.decimalPoint ? ",DecimalPoint" : "") + ")";
        }
    }
    
    static final class TextPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TemporalField field;
        private final TextStyle textStyle;
        private final DateTimeTextProvider provider;
        private volatile DateTimeFormatterBuilder.NumberPrinterParser numberPrinterParser;
        
        TextPrinterParser(TemporalField field, TextStyle textStyle, DateTimeTextProvider provider) {
            this.field = field;
            this.textStyle = textStyle;
            this.provider = provider;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(this.field);
            if (value == null) {
                return false;
            }
            Chronology chronology = (Chronology)dateTimePrintContext.getTemporal().query(TemporalQueries.chronology());
            String str;
            if (chronology == null || chronology == IsoChronology.INSTANCE) {
                str = this.provider.getText(this.field, (long)value, this.textStyle, dateTimePrintContext.getLocale());
            }
            else {
                str = this.provider.getText(chronology, this.field, (long)value, this.textStyle, dateTimePrintContext.getLocale());
            }
            if (str == null) {
                return this.numberPrinterParser().format(dateTimePrintContext, sb);
            }
            sb.append(str);
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            int length = charSequence.length();
            if (n < 0 || n > length) {
                throw new IndexOutOfBoundsException();
            }
            TextStyle textStyle = dateTimeParseContext.isStrict() ? this.textStyle : null;
            Chronology effectiveChronology = dateTimeParseContext.getEffectiveChronology();
            Iterator<Map.Entry<String, Long>> iterator;
            if (effectiveChronology == null || effectiveChronology == IsoChronology.INSTANCE) {
                iterator = this.provider.getTextIterator(this.field, textStyle, dateTimeParseContext.getLocale());
            }
            else {
                iterator = this.provider.getTextIterator(effectiveChronology, this.field, textStyle, dateTimeParseContext.getLocale());
            }
            if (iterator != null) {
                while (iterator.hasNext()) {
                    Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
                    String s = (String)entry.getKey();
                    if (dateTimeParseContext.subSequenceEquals(s, 0, charSequence, n, s.length())) {
                        return dateTimeParseContext.setParsedField(this.field, (long)entry.getValue(), n, n + s.length());
                    }
                    continue;
                }
                if (dateTimeParseContext.isStrict()) {
                    return ~n;
                }
            }
            return this.numberPrinterParser().parse(dateTimeParseContext, charSequence, n);
        }
        
        private DateTimeFormatterBuilder.NumberPrinterParser numberPrinterParser() {
            if (this.numberPrinterParser == null) {
                this.numberPrinterParser = new DateTimeFormatterBuilder.NumberPrinterParser(this.field, 1, 19, SignStyle.NORMAL);
            }
            return this.numberPrinterParser;
        }
        
        @Override
        public String toString() {
            if (this.textStyle == TextStyle.FULL) {
                return "Text(" + this.field + ")";
            }
            return "Text(" + this.field + "," + this.textStyle + ")";
        }
    }
    
    static final class InstantPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private static final long SECONDS_PER_10000_YEARS = 315569520000L;
        private static final long SECONDS_0000_TO_1970 = 62167219200L;
        private final int fractionalDigits;
        
        InstantPrinterParser(int fractionalDigits) {
            this.fractionalDigits = fractionalDigits;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(ChronoField.INSTANT_SECONDS);
            Long value2 = null;
            if (dateTimePrintContext.getTemporal().isSupported(ChronoField.NANO_OF_SECOND)) {
                value2 = Long.valueOf(dateTimePrintContext.getTemporal().getLong(ChronoField.NANO_OF_SECOND));
            }
            if (value == null) {
                return false;
            }
            long longValue = (long)value;
            int checkValidIntValue = ChronoField.NANO_OF_SECOND.checkValidIntValue((value2 != null) ? ((long)value2) : 0L);
            if (longValue >= -62167219200L) {
                long n4 = longValue - 315569520000L + 62167219200L;
                long lng = Math.floorDiv(n4, 315569520000L) + 1L;
                LocalDateTime ofEpochSecond2 = LocalDateTime.ofEpochSecond(Math.floorMod(n4, 315569520000L) - 62167219200L, 0, ZoneOffset.UTC);
                if (lng > 0L) {
                    sb.append('+').append(lng);
                }
                sb.append(ofEpochSecond2);
                if (ofEpochSecond2.getSecond() == 0) {
                    sb.append(":00");
                }
            }
            else {
                long n = longValue + 62167219200L;
                long n2 = n / 315569520000L;
                long n3 = n % 315569520000L;
                LocalDateTime ofEpochSecond = LocalDateTime.ofEpochSecond(n3 - 62167219200L, 0, ZoneOffset.UTC);
                int length = sb.length();
                sb.append(ofEpochSecond);
                if (ofEpochSecond.getSecond() == 0) {
                    sb.append(":00");
                }
                if (n2 < 0L) {
                    if (ofEpochSecond.getYear() == -10000) {
                        sb.replace(length, length + 2, Long.toString(n2 - 1L));
                    }
                    else if (n3 == 0L) {
                        sb.insert(length, n2);
                    }
                    else {
                        sb.insert(length + 1, Math.abs(n2));
                    }
                }
            }
            if ((this.fractionalDigits < 0 && checkValidIntValue > 0) || this.fractionalDigits > 0) {
                sb.append('.');
                int n7;
                for (int n5 = 100000000, n6 = 0; (this.fractionalDigits == -1 && checkValidIntValue > 0) || (this.fractionalDigits == -2 && (checkValidIntValue > 0 || n6 % 3 != 0)) || n6 < this.fractionalDigits; checkValidIntValue -= n7 * n5, n5 /= 10, ++n6) {
                    n7 = checkValidIntValue / n5;
                    sb.append((char)(n7 + 48));
                }
            }
            sb.append('Z');
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            DateTimeFormatterBuilder.CompositePrinterParser printerParser = new DateTimeFormatterBuilder().append(DateTimeFormatter.ISO_LOCAL_DATE).appendLiteral('T').appendValue(ChronoField.HOUR_OF_DAY, 2).appendLiteral(':').appendValue(ChronoField.MINUTE_OF_HOUR, 2).appendLiteral(':').appendValue(ChronoField.SECOND_OF_MINUTE, 2).appendFraction(ChronoField.NANO_OF_SECOND, (this.fractionalDigits < 0) ? 0 : this.fractionalDigits, (this.fractionalDigits < 0) ? 9 : this.fractionalDigits, true).appendLiteral('Z').toFormatter().toPrinterParser(false);
            DateTimeParseContext copy = dateTimeParseContext.copy();
            int parse = printerParser.parse(copy, charSequence, n);
            if (parse < 0) {
                return parse;
            }
            long longValue = (long)copy.getParsed(ChronoField.YEAR);
            int intValue = copy.getParsed(ChronoField.MONTH_OF_YEAR).intValue();
            int intValue2 = copy.getParsed(ChronoField.DAY_OF_MONTH).intValue();
            int intValue3 = copy.getParsed(ChronoField.HOUR_OF_DAY).intValue();
            int intValue4 = copy.getParsed(ChronoField.MINUTE_OF_HOUR).intValue();
            Long parsed = copy.getParsed(ChronoField.SECOND_OF_MINUTE);
            Long parsed2 = copy.getParsed(ChronoField.NANO_OF_SECOND);
            int second = (parsed != null) ? parsed.intValue() : 0;
            int n2 = (parsed2 != null) ? parsed2.intValue() : 0;
            boolean n3 = false;
            if (intValue3 == 24 && intValue4 == 0 && second == 0 && n2 == 0) {
                intValue3 = 0;
                n3 = true;
            }
            else if (intValue3 == 23 && intValue4 == 59 && second == 60) {
                dateTimeParseContext.setParsedLeapSecond();
                second = 59;
            }
            int year = (int)longValue % 10000;
            long n4;
            try {
                n4 = LocalDateTime.of(year, intValue, intValue2, intValue3, intValue4, second, 0).plusDays((long)n3).toEpochSecond(ZoneOffset.UTC) + Math.multiplyExact(longValue / 10000L, 315569520000L);
            }
            catch (RuntimeException ex) {
                return ~n;
            }
            return dateTimeParseContext.setParsedField(ChronoField.NANO_OF_SECOND, (long)n2, n, dateTimeParseContext.setParsedField(ChronoField.INSTANT_SECONDS, n4, n, parse));
        }
        
        @Override
        public String toString() {
            return "Instant()";
        }
    }
    
    static final class OffsetIdPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        static final String[] PATTERNS;
        static final DateTimeFormatterBuilder.OffsetIdPrinterParser INSTANCE_ID_Z;
        static final DateTimeFormatterBuilder.OffsetIdPrinterParser INSTANCE_ID_ZERO;
        private final String noOffsetText;
        private final int type;
        
        OffsetIdPrinterParser(String obj, String s) {
            Objects.requireNonNull(obj, "pattern");
            Objects.requireNonNull(s, "noOffsetText");
            this.type = this.checkPattern(obj);
            this.noOffsetText = s;
        }
        
        private int checkPattern(String s) {
            int i = 0;
            while (i < DateTimeFormatterBuilder.OffsetIdPrinterParser.PATTERNS.length) {
                if (DateTimeFormatterBuilder.OffsetIdPrinterParser.PATTERNS[i].equals(s)) {
                    return i;
                }
                ++i;
            }
            throw new IllegalArgumentException("Invalid zone offset pattern: " + s);
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(ChronoField.OFFSET_SECONDS);
            if (value == null) {
                return false;
            }
            int intExact = Math.toIntExact((long)value);
            if (intExact == 0) {
                sb.append(this.noOffsetText);
            }
            else {
                int abs = Math.abs(intExact / 3600 % 100);
                int abs2 = Math.abs(intExact / 60 % 60);
                int abs3 = Math.abs(intExact % 60);
                int length = sb.length();
                sb.append((intExact < 0) ? "-" : "+").append((char)(abs / 10 + 48)).append((char)(abs % 10 + 48));
                if (this.type >= 3 || (this.type >= 1 && abs2 > 0)) {
                    sb.append((this.type % 2 == 0) ? ":" : "").append((char)(abs2 / 10 + 48)).append((char)(abs2 % 10 + 48));
                    abs += abs2;
                    if (this.type >= 7 || (this.type >= 5 && abs3 > 0)) {
                        sb.append((this.type % 2 == 0) ? ":" : "").append((char)(abs3 / 10 + 48)).append((char)(abs3 % 10 + 48));
                        abs += abs3;
                    }
                }
                if (abs == 0) {
                    sb.setLength(length);
                    sb.append(this.noOffsetText);
                }
            }
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            int length = charSequence.length();
            int length2 = this.noOffsetText.length();
            if (length2 == 0) {
                if (n == length) {
                    return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, 0L, n, n);
                }
            }
            else {
                if (n == length) {
                    return ~n;
                }
                if (dateTimeParseContext.subSequenceEquals(charSequence, n, this.noOffsetText, 0, length2)) {
                    return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, 0L, n, n + length2);
                }
            }
            int char1 = charSequence.charAt(n);
            if (char1 == '+' || char1 == '-') {
                int n2 = (char1 == '-') ? -1 : 1;
                int[] array = new int[4];
                array[0] = n + 1;
                if (!this.parseNumber(array, 1, charSequence, true) && !this.parseNumber(array, 2, charSequence, this.type >= 3) && !this.parseNumber(array, 3, charSequence, false)) {
                    return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, (long)n2 * ((long)array[1] * 3600L + (long)array[2] * 60L + (long)array[3]), n, array[0]);
                }
            }
            if (length2 == 0) {
                return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, 0L, n, n + length2);
            }
            return ~n;
        }
        
        private boolean parseNumber(int[] array, int n, CharSequence charSequence, boolean b) {
            if ((this.type + 3) / 2 < n) {
                return false;
            }
            int n2 = array[0];
            if (this.type % 2 == 0 && n > 1) {
                if (n2 + 1 > charSequence.length() || charSequence.charAt(n2) != ':') {
                    return b;
                }
                ++n2;
            }
            if (n2 + 2 > charSequence.length()) {
                return b;
            }
            int char1 = charSequence.charAt(n2++);
            int char2 = charSequence.charAt(n2++);
            if (char1 < '0' || char1 > '9' || char2 < '0' || char2 > '9') {
                return b;
            }
            int n3 = (char1 - 48) * 10 + (char2 - 48);
            if (n3 < 0 || n3 > 59) {
                return b;
            }
            array[n] = n3;
            array[0] = n2;
            return false;
        }
        
        @Override
        public String toString() {
            return "Offset(" + DateTimeFormatterBuilder.OffsetIdPrinterParser.PATTERNS[this.type] + ",'" + this.noOffsetText.replace("'", "''") + "')";
        }
        
        static {
            PATTERNS = new String[] { "+HH", "+HHmm", "+HH:mm", "+HHMM", "+HH:MM", "+HHMMss", "+HH:MM:ss", "+HHMMSS", "+HH:MM:SS" };
            INSTANCE_ID_Z = new DateTimeFormatterBuilder.OffsetIdPrinterParser("+HH:MM:ss", "Z");
            INSTANCE_ID_ZERO = new DateTimeFormatterBuilder.OffsetIdPrinterParser("+HH:MM:ss", "0");
        }
    }
    
    static final class LocalizedOffsetIdPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TextStyle style;
        
        LocalizedOffsetIdPrinterParser(TextStyle style) {
            this.style = style;
        }
        
        private static StringBuilder appendHMS(StringBuilder sb, int n) {
            return sb.append((char)(n / 10 + 48)).append((char)(n % 10 + 48));
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Long value = dateTimePrintContext.getValue(ChronoField.OFFSET_SECONDS);
            if (value == null) {
                return false;
            }
            String str = "GMT";
            if (str != null) {
                sb.append(str);
            }
            int intExact = Math.toIntExact((long)value);
            if (intExact != 0) {
                int abs = Math.abs(intExact / 3600 % 100);
                int abs2 = Math.abs(intExact / 60 % 60);
                int abs3 = Math.abs(intExact % 60);
                sb.append((intExact < 0) ? "-" : "+");
                if (this.style == TextStyle.FULL) {
                    appendHMS(sb, abs);
                    sb.append(':');
                    appendHMS(sb, abs2);
                    if (abs3 != 0) {
                        sb.append(':');
                        appendHMS(sb, abs3);
                    }
                }
                else {
                    if (abs >= 10) {
                        sb.append((char)(abs / 10 + 48));
                    }
                    sb.append((char)(abs % 10 + 48));
                    if (abs2 != 0 || abs3 != 0) {
                        sb.append(':');
                        appendHMS(sb, abs2);
                        if (abs3 != 0) {
                            sb.append(':');
                            appendHMS(sb, abs3);
                        }
                    }
                }
            }
            return true;
        }
        
        int getDigit(CharSequence charSequence, int n) {
            int char1 = charSequence.charAt(n);
            if (char1 < '0' || char1 > '9') {
                return -1;
            }
            return char1 - 48;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            int n2 = n + charSequence.length();
            String s = "GMT";
            if (s != null) {
                if (!dateTimeParseContext.subSequenceEquals(charSequence, n, s, 0, s.length())) {
                    return ~n;
                }
                n += s.length();
            }
            if (n == n2) {
                return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, 0L, n, n);
            }
            int char1 = charSequence.charAt(n);
            int n3;
            if (char1 == '+') {
                n3 = 1;
            }
            else if (char1 == '-') {
                n3 = -1;
            }
            else {
                return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, 0L, n, n);
            }
            ++n;
            int n4 = 0;
            int n5 = 0;
            int digit;
            if (this.style == TextStyle.FULL) {
                int digit7 = this.getDigit(charSequence, n++);
                int digit8 = this.getDigit(charSequence, n++);
                if (digit7 < 0 || digit8 < 0 || charSequence.charAt(n++) != ':') {
                    return ~n;
                }
                digit = digit7 * 10 + digit8;
                int digit9 = this.getDigit(charSequence, n++);
                int digit10 = this.getDigit(charSequence, n++);
                if (digit9 < 0 || digit10 < 0) {
                    return ~n;
                }
                n4 = digit9 * 10 + digit10;
                if (n + 2 < n2 && charSequence.charAt(n) == ':') {
                    int digit11 = this.getDigit(charSequence, n + 1);
                    int digit12 = this.getDigit(charSequence, n + 2);
                    if (digit11 >= 0 && digit12 >= 0) {
                        n5 = digit11 * 10 + digit12;
                        n += 3;
                    }
                }
            }
            else {
                digit = this.getDigit(charSequence, n++);
                if (digit < 0) {
                    return ~n;
                }
                if (n < n2) {
                    int digit2 = this.getDigit(charSequence, n);
                    if (digit2 >= 0) {
                        digit = digit * 10 + digit2;
                        ++n;
                    }
                    if (n + 2 < n2 && charSequence.charAt(n) == ':' && n + 2 < n2 && charSequence.charAt(n) == ':') {
                        int digit3 = this.getDigit(charSequence, n + 1);
                        int digit4 = this.getDigit(charSequence, n + 2);
                        if (digit3 >= 0 && digit4 >= 0) {
                            n4 = digit3 * 10 + digit4;
                            n += 3;
                            if (n + 2 < n2 && charSequence.charAt(n) == ':') {
                                int digit5 = this.getDigit(charSequence, n + 1);
                                int digit6 = this.getDigit(charSequence, n + 2);
                                if (digit5 >= 0 && digit6 >= 0) {
                                    n5 = digit5 * 10 + digit6;
                                    n += 3;
                                }
                            }
                        }
                    }
                }
            }
            return dateTimeParseContext.setParsedField(ChronoField.OFFSET_SECONDS, (long)n3 * ((long)digit * 3600L + (long)n4 * 60L + (long)n5), n, n);
        }
        
        @Override
        public String toString() {
            return "LocalizedOffset(" + this.style + ")";
        }
    }
    
    static final class ZoneTextPrinterParser extends DateTimeFormatterBuilder.ZoneIdPrinterParser
    {
        private final TextStyle textStyle;
        private Set<String> preferredZones;
        private static final int STD = 0;
        private static final int DST = 1;
        private static final int GENERIC = 2;
        private static final Map<String, SoftReference<Map<Locale, String[]>>> cache;
        private final Map<Locale, Map.Entry<Integer, SoftReference<DateTimeFormatterBuilder.PrefixTree>>> cachedTree;
        private final Map<Locale, Map.Entry<Integer, SoftReference<DateTimeFormatterBuilder.PrefixTree>>> cachedTreeCI;
        
        ZoneTextPrinterParser(TextStyle textStyle, Set<ZoneId> set) {
            super(TemporalQueries.zone(), "ZoneText(" + textStyle + ")");
            this.cachedTree = new HashMap();
            this.cachedTreeCI = new HashMap();
            this.textStyle = (TextStyle)Objects.requireNonNull(textStyle, "textStyle");
            if (set != null && set.size() != 0) {
                this.preferredZones = new HashSet();
                Iterator<Object> iterator = set.iterator();
                while (iterator.hasNext()) {
                    this.preferredZones.add(((ZoneId)iterator.next()).getId());
                }
            }
        }
        
        private String getDisplayName(String id, int n, Locale locale) {
            if (this.textStyle == TextStyle.NARROW) {
                return null;
            }
            SoftReference<Object> softReference = (SoftReference)DateTimeFormatterBuilder.ZoneTextPrinterParser.cache.get(id);
            Object referent = null;
            String[] array;
            if (softReference == null || (referent = (Object)softReference.get()) == null || (array = (String[])referent.get(locale)) == null) {
                String[] retrieveDisplayNames = TimeZoneNameUtility.retrieveDisplayNames(id, locale);
                if (retrieveDisplayNames == null) {
                    return null;
                }
                array = (String[])Arrays.copyOfRange(retrieveDisplayNames, 0, 7);
                array[5] = TimeZoneNameUtility.retrieveGenericDisplayName(id, 1, locale);
                if (array[5] == null) {
                    array[5] = array[0];
                }
                array[6] = TimeZoneNameUtility.retrieveGenericDisplayName(id, 0, locale);
                if (array[6] == null) {
                    array[6] = array[0];
                }
                if (referent == null) {
                    referent = new ConcurrentHashMap();
                }
                referent.put(locale, array);
                DateTimeFormatterBuilder.ZoneTextPrinterParser.cache.put(id, new SoftReference(referent));
            }
            switch (n) {
                case 0: {
                    return array[this.textStyle.zoneNameStyleIndex() + 1];
                }
                case 1: {
                    return array[this.textStyle.zoneNameStyleIndex() + 3];
                }
                default: {
                    return array[this.textStyle.zoneNameStyleIndex() + 5];
                }
            }
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            ZoneId zoneId = (ZoneId)dateTimePrintContext.getValue(TemporalQueries.zoneId());
            if (zoneId == null) {
                return false;
            }
            String id = zoneId.getId();
            if (!(zoneId instanceof ZoneOffset)) {
                TemporalAccessor temporal = dateTimePrintContext.getTemporal();
                String displayName = this.getDisplayName(id, temporal.isSupported(ChronoField.INSTANT_SECONDS) ? (zoneId.getRules().isDaylightSavings(Instant.from(temporal)) ? 1 : 0) : 2, dateTimePrintContext.getLocale());
                if (displayName != null) {
                    id = displayName;
                }
            }
            sb.append(id);
            return true;
        }
        
        @Override
        protected DateTimeFormatterBuilder.PrefixTree getTree(DateTimeParseContext dateTimeParseContext) {
            if (this.textStyle == TextStyle.NARROW) {
                return super.getTree(dateTimeParseContext);
            }
            Locale locale = dateTimeParseContext.getLocale();
            int caseSensitive = dateTimeParseContext.isCaseSensitive();
            Set<String> availableZoneIds = ZoneRulesProvider.getAvailableZoneIds();
            int size = availableZoneIds.size();
            Map<Locale, AbstractMap.SimpleImmutableEntry<Integer, Object>> map = caseSensitive ? this.cachedTree : this.cachedTreeCI;
            String[][] array = (String[][])null;
            Map.Entry<Object, Object> entry;
            DateTimeFormatterBuilder.PrefixTree tree;
            if ((entry = (Map.Entry)map.get(locale)) == null || (int)entry.getKey() != size || (tree = (DateTimeFormatterBuilder.PrefixTree)((SoftReference)entry.getValue()).get()) == null) {
                tree = DateTimeFormatterBuilder.PrefixTree.newTree(dateTimeParseContext);
                String[][] zoneStrings = TimeZoneNameUtility.getZoneStrings(locale);
                for (String[] array2 : zoneStrings) {
                    String s = array2[0];
                    if (availableZoneIds.contains(s)) {
                        tree.add(s, s);
                        String zid = ZoneName.toZid(s, locale);
                        for (int j = (this.textStyle == TextStyle.FULL) ? 1 : 2; j < array2.length; j += 2) {
                            tree.add(array2[j], zid);
                        }
                    }
                }
                if (this.preferredZones != null) {
                    for (String[] array3 : zoneStrings) {
                        String s2 = array3[0];
                        if (this.preferredZones.contains(s2) && availableZoneIds.contains(s2)) {
                            for (int l = (this.textStyle == TextStyle.FULL) ? 1 : 2; l < array3.length; l += 2) {
                                tree.add(array3[l], s2);
                            }
                        }
                    }
                }
                map.put(locale, new AbstractMap.SimpleImmutableEntry(Integer.valueOf(size), new SoftReference(tree)));
                return tree;
            }
            else {
                return tree;
            }
        }
        
        static {
            cache = new ConcurrentHashMap();
        }
    }
    
    static class ZoneIdPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TemporalQuery<ZoneId> query;
        private final String description;
        private static volatile Map.Entry<Integer, DateTimeFormatterBuilder.PrefixTree> cachedPrefixTree;
        private static volatile Map.Entry<Integer, DateTimeFormatterBuilder.PrefixTree> cachedPrefixTreeCI;
        
        ZoneIdPrinterParser(TemporalQuery<ZoneId> query, String description) {
            this.query = query;
            this.description = description;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            ZoneId zoneId = (ZoneId)dateTimePrintContext.getValue(this.query);
            if (zoneId == null) {
                return false;
            }
            sb.append(zoneId.getId());
            return true;
        }
        
        protected DateTimeFormatterBuilder.PrefixTree getTree(DateTimeParseContext dateTimeParseContext) {
            Set<String> availableZoneIds = ZoneRulesProvider.getAvailableZoneIds();
            int size = availableZoneIds.size();
            Map.Entry<Object, Object> entry = dateTimeParseContext.isCaseSensitive() ? DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTree : DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTreeCI;
            if (entry == null || (int)entry.getKey() != size) {
                synchronized (this) {
                    entry = (dateTimeParseContext.isCaseSensitive() ? DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTree : DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTreeCI);
                    if (entry == null || (int)entry.getKey() != size) {
                        entry = new AbstractMap.SimpleImmutableEntry(Integer.valueOf(size), DateTimeFormatterBuilder.PrefixTree.newTree(availableZoneIds, dateTimeParseContext));
                        if (dateTimeParseContext.isCaseSensitive()) {
                            DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTree = entry;
                        }
                        else {
                            DateTimeFormatterBuilder.ZoneIdPrinterParser.cachedPrefixTreeCI = entry;
                        }
                    }
                }
            }
            return (DateTimeFormatterBuilder.PrefixTree)entry.getValue();
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int index) {
            int length = charSequence.length();
            if (index > length) {
                throw new IndexOutOfBoundsException();
            }
            if (index == length) {
                return ~index;
            }
            int char1 = charSequence.charAt(index);
            if (char1 == '+' || char1 == '-') {
                return this.parseOffsetBased(dateTimeParseContext, charSequence, index, index, DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_Z);
            }
            if (length >= index + 2) {
                int char2 = charSequence.charAt(index + 1);
                if (dateTimeParseContext.charEquals((char)char1, 'U') && dateTimeParseContext.charEquals((char)char2, 'T')) {
                    if (length >= index + 3 && dateTimeParseContext.charEquals(charSequence.charAt(index + 2), 'C')) {
                        return this.parseOffsetBased(dateTimeParseContext, charSequence, index, index + 3, DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_ZERO);
                    }
                    return this.parseOffsetBased(dateTimeParseContext, charSequence, index, index + 2, DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_ZERO);
                }
                else if (dateTimeParseContext.charEquals((char)char1, 'G') && length >= index + 3 && dateTimeParseContext.charEquals((char)char2, 'M') && dateTimeParseContext.charEquals(charSequence.charAt(index + 2), 'T')) {
                    return this.parseOffsetBased(dateTimeParseContext, charSequence, index, index + 3, DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_ZERO);
                }
            }
            DateTimeFormatterBuilder.PrefixTree tree = this.getTree(dateTimeParseContext);
            ParsePosition parsePosition = new ParsePosition(index);
            String match = tree.match(charSequence, parsePosition);
            if (match == null) {
                if (dateTimeParseContext.charEquals((char)char1, 'Z')) {
                    dateTimeParseContext.setParsed(ZoneOffset.UTC);
                    return index + 1;
                }
                return ~index;
            }
            else {
                dateTimeParseContext.setParsed(ZoneId.of(match));
                return parsePosition.getIndex();
            }
        }
        
        private int parseOffsetBased(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int beginIndex, int endIndex, DateTimeFormatterBuilder.OffsetIdPrinterParser offsetIdPrinterParser) {
            String upperCase = charSequence.toString().substring(beginIndex, endIndex).toUpperCase();
            if (endIndex >= charSequence.length()) {
                dateTimeParseContext.setParsed(ZoneId.of(upperCase));
                return endIndex;
            }
            if (charSequence.charAt(endIndex) == '0' || dateTimeParseContext.charEquals(charSequence.charAt(endIndex), 'Z')) {
                dateTimeParseContext.setParsed(ZoneId.of(upperCase));
                return endIndex;
            }
            DateTimeParseContext copy = dateTimeParseContext.copy();
            int parse = offsetIdPrinterParser.parse(copy, charSequence, endIndex);
            try {
                if (parse < 0) {
                    if (offsetIdPrinterParser == DateTimeFormatterBuilder.OffsetIdPrinterParser.INSTANCE_ID_Z) {
                        return ~beginIndex;
                    }
                    dateTimeParseContext.setParsed(ZoneId.of(upperCase));
                    return endIndex;
                }
                else {
                    dateTimeParseContext.setParsed(ZoneId.ofOffset(upperCase, ZoneOffset.ofTotalSeconds((int)(long)copy.getParsed(ChronoField.OFFSET_SECONDS))));
                    return parse;
                }
            }
            catch (DateTimeException ex) {
                return ~beginIndex;
            }
        }
        
        @Override
        public String toString() {
            return this.description;
        }
    }
    
    static class PrefixTree
    {
        protected String key;
        protected String value;
        protected char c0;
        protected DateTimeFormatterBuilder.PrefixTree child;
        protected DateTimeFormatterBuilder.PrefixTree sibling;
        
        private PrefixTree(String key, String value, DateTimeFormatterBuilder.PrefixTree child) {
            this.key = key;
            this.value = value;
            this.child = child;
            if (key.length() == 0) {
                this.c0 = '\uffff';
            }
            else {
                this.c0 = this.key.charAt(0);
            }
        }
        
        public static DateTimeFormatterBuilder.PrefixTree newTree(DateTimeParseContext dateTimeParseContext) {
            if (dateTimeParseContext.isCaseSensitive()) {
                return new DateTimeFormatterBuilder.PrefixTree("", null);
            }
            return new DateTimeFormatterBuilder.PrefixTree.CI("", null);
        }
        
        public static DateTimeFormatterBuilder.PrefixTree newTree(Set<String> set, DateTimeParseContext dateTimeParseContext) {
            DateTimeFormatterBuilder.PrefixTree tree = newTree(dateTimeParseContext);
            for (String s : set) {
                tree.add0(s, s);
            }
            return tree;
        }
        
        public DateTimeFormatterBuilder.PrefixTree copyTree() {
            DateTimeFormatterBuilder.PrefixTree prefixTree = new DateTimeFormatterBuilder.PrefixTree(this.key, this.value);
            if (this.child != null) {
                prefixTree.child = this.child.copyTree();
            }
            if (this.sibling != null) {
                prefixTree.sibling = this.sibling.copyTree();
                return prefixTree;
            }
            return prefixTree;
        }
        
        public boolean add(String s, String s2) {
            return this.add0(s, s2);
        }
        
        private boolean add0(String key, String s) {
            key = this.toKey(key);
            int prefixLength = this.prefixLength(key);
            if (prefixLength == this.key.length()) {
                if (prefixLength < key.length()) {
                    String substring = key.substring(prefixLength);
                    DateTimeFormatterBuilder.PrefixTree prefixTree = this.child;
                    while (prefixTree != null) {
                        if (this.isEqual(prefixTree.c0, substring.charAt(0))) {
                            return prefixTree.add0(substring, s);
                        }
                        prefixTree = prefixTree.sibling;
                    }
                    DateTimeFormatterBuilder.PrefixTree node2 = this.newNode(substring, s, null);
                    node2.sibling = this.child;
                    this.child = node2;
                    return true;
                }
                this.value = s;
                return true;
            }
            else {
                DateTimeFormatterBuilder.PrefixTree node = this.newNode(this.key.substring(prefixLength), this.value, this.child);
                this.key = key.substring(0, prefixLength);
                this.child = node;
                if (prefixLength < key.length()) {
                    this.child.sibling = this.newNode(key.substring(prefixLength), s, null);
                    this.value = null;
                }
                else {
                    this.value = s;
                }
                return true;
            }
        }
        
        public String match(CharSequence charSequence, int n, int n2) {
            if (!this.prefixOf(charSequence, n, n2)) {
                return null;
            }
            if (this.child != null && (n += this.key.length()) != n2) {
                DateTimeFormatterBuilder.PrefixTree prefixTree = this.child;
                while (!this.isEqual(prefixTree.c0, charSequence.charAt(n))) {
                    prefixTree = prefixTree.sibling;
                    if (prefixTree == null) {
                        return this.value;
                    }
                    continue;
                }
                String match = prefixTree.match(charSequence, n, n2);
                if (match != null) {
                    return match;
                }
                return this.value;
            }
            return this.value;
        }
        
        public String match(CharSequence charSequence, ParsePosition parsePosition) {
            int index = parsePosition.getIndex();
            int length = charSequence.length();
            if (!this.prefixOf(charSequence, index, length)) {
                return null;
            }
            int n = index + this.key.length();
            Label_0108: {
                if (this.child != null && n != length) {
                    DateTimeFormatterBuilder.PrefixTree prefixTree = this.child;
                    while (!this.isEqual(prefixTree.c0, charSequence.charAt(n))) {
                        prefixTree = prefixTree.sibling;
                        if (prefixTree == null) {
                            break Label_0108;
                        }
                        continue;
                    }
                    parsePosition.setIndex(n);
                    String match = prefixTree.match(charSequence, parsePosition);
                    if (match != null) {
                        return match;
                    }
                }
            }
            parsePosition.setIndex(n);
            return this.value;
        }
        
        protected String toKey(String s) {
            return s;
        }
        
        protected DateTimeFormatterBuilder.PrefixTree newNode(String s, String s2, DateTimeFormatterBuilder.PrefixTree prefixTree) {
            return new DateTimeFormatterBuilder.PrefixTree(s, s2, prefixTree);
        }
        
        protected boolean isEqual(char c, char c2) {
            return c == c2;
        }
        
        protected boolean prefixOf(CharSequence charSequence, int toffset, int n) {
            if ((charSequence instanceof String)) {
                return ((String)charSequence).startsWith(this.key, toffset);
            }
            int length = this.key.length();
            if (length > n - toffset) {
                return false;
            }
            int n2 = 0;
            while (length-- > 0) {
                if (this.isEqual(this.key.charAt(n2++), charSequence.charAt(toffset++))) {
                    continue;
                }
                return false;
            }
            return true;
        }
        
        private int prefixLength(String s) {
            int n = 0;
            while (n < s.length() && n < this.key.length()) {
                if (!this.isEqual(s.charAt(n), this.key.charAt(n))) {
                    return n;
                }
                ++n;
            }
            return n;
        }
        
        private static class CI extends DateTimeFormatterBuilder.PrefixTree
        {
            private CI(String s, String s2, DateTimeFormatterBuilder.PrefixTree prefixTree) {
                super(s, s2, prefixTree);
            }
            
            @Override
            protected DateTimeFormatterBuilder.PrefixTree.CI newNode(String s, String s2, DateTimeFormatterBuilder.PrefixTree prefixTree) {
                return new DateTimeFormatterBuilder.PrefixTree.CI(s, s2, prefixTree);
            }
            
            @Override
            protected boolean isEqual(char c, char c2) {
                return DateTimeParseContext.charEqualsIgnoreCase(c, c2);
            }
            
            @Override
            protected boolean prefixOf(CharSequence charSequence, int n, int n2) {
                int length = this.key.length();
                if (length > n2 - n) {
                    return false;
                }
                int n3 = 0;
                while (length-- > 0) {
                    if (this.isEqual(this.key.charAt(n3++), charSequence.charAt(n++))) {
                        continue;
                    }
                    return false;
                }
                return true;
            }
        }
        
        private static class LENIENT extends DateTimeFormatterBuilder.PrefixTree.CI
        {
            private LENIENT(String s, String s2, DateTimeFormatterBuilder.PrefixTree prefixTree) {
                super(s, s2, prefixTree);
            }
            
            @Override
            protected DateTimeFormatterBuilder.PrefixTree.CI newNode(String s, String s2, DateTimeFormatterBuilder.PrefixTree prefixTree) {
                return new DateTimeFormatterBuilder.PrefixTree.LENIENT(s, s2, prefixTree);
            }
            
            private boolean isLenientChar(char c) {
                return c == ' ' || c == '_' || c == '/';
            }
            
            @Override
            protected String toKey(String s) {
                int i = 0;
                while (i < s.length()) {
                    if (this.isLenientChar(s.charAt(i))) {
                        StringBuilder sb = new StringBuilder(s.length());
                        sb.append(s, 0, i);
                        ++i;
                        while (i < s.length()) {
                            if (!this.isLenientChar(s.charAt(i))) {
                                sb.append(s.charAt(i));
                            }
                            ++i;
                        }
                        return sb.toString();
                    }
                    ++i;
                }
                return s;
            }
            
            @Override
            public String match(CharSequence charSequence, ParsePosition parsePosition) {
                int index = parsePosition.getIndex();
                int length = charSequence.length();
                int length2 = this.key.length();
                int n = 0;
                while (n < length2 && index < length) {
                    if (this.isLenientChar(charSequence.charAt(index))) {
                        ++index;
                    }
                    else {
                        if (this.isEqual(this.key.charAt(n++), charSequence.charAt(index++))) {
                            continue;
                        }
                        return null;
                    }
                }
                if (n != length2) {
                    return null;
                }
                Label_0210: {
                    if (this.child != null && index != length) {
                        for (; index < length && this.isLenientChar(charSequence.charAt(index)); ++index) {}
                        if (index < length) {
                            DateTimeFormatterBuilder.PrefixTree prefixTree = this.child;
                            while (!this.isEqual(prefixTree.c0, charSequence.charAt(index))) {
                                prefixTree = prefixTree.sibling;
                                if (prefixTree == null) {
                                    break Label_0210;
                                }
                                continue;
                            }
                            parsePosition.setIndex(index);
                            String match = prefixTree.match(charSequence, parsePosition);
                            if (match != null) {
                                return match;
                            }
                        }
                    }
                }
                parsePosition.setIndex(index);
                return this.value;
            }
        }
    }
    
    static final class ChronoPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private final TextStyle textStyle;
        
        ChronoPrinterParser(TextStyle textStyle) {
            this.textStyle = textStyle;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            Chronology chronology = (Chronology)dateTimePrintContext.getValue(TemporalQueries.chronology());
            if (chronology == null) {
                return false;
            }
            if (this.textStyle == null) {
                sb.append(chronology.getId());
            }
            else {
                sb.append(this.getChronologyName(chronology, dateTimePrintContext.getLocale()));
            }
            return true;
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            if (n < 0 || n > charSequence.length()) {
                throw new IndexOutOfBoundsException();
            }
            Set<Chronology> availableChronologies = Chronology.getAvailableChronologies();
            Chronology parsed = null;
            int n2 = -1;
            for (Chronology chronology : availableChronologies) {
                String s;
                if (this.textStyle == null) {
                    s = chronology.getId();
                }
                else {
                    s = this.getChronologyName(chronology, dateTimeParseContext.getLocale());
                }
                int length = s.length();
                if (length > n2 && dateTimeParseContext.subSequenceEquals(charSequence, n, s, 0, length)) {
                    parsed = chronology;
                    n2 = length;
                }
                else {
                    continue;
                }
            }
            if (parsed == null) {
                return ~n;
            }
            dateTimeParseContext.setParsed(parsed);
            return n + n2;
        }
        
        private String getChronologyName(Chronology chronology, Locale locale) {
            String s = (String)DateTimeTextProvider.getLocalizedResource("calendarname." + chronology.getCalendarType(), locale);
            return (s != null) ? s : chronology.getId();
        }
    }
    
    static final class LocalizedPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private static final ConcurrentMap<String, DateTimeFormatter> FORMATTER_CACHE;
        private final FormatStyle dateStyle;
        private final FormatStyle timeStyle;
        
        LocalizedPrinterParser(FormatStyle dateStyle, FormatStyle timeStyle) {
            this.dateStyle = dateStyle;
            this.timeStyle = timeStyle;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            return this.formatter(dateTimePrintContext.getLocale(), Chronology.from(dateTimePrintContext.getTemporal())).toPrinterParser(false).format(dateTimePrintContext, sb);
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            return this.formatter(dateTimeParseContext.getLocale(), dateTimeParseContext.getEffectiveChronology()).toPrinterParser(false).parse(dateTimeParseContext, charSequence, n);
        }
        
        private DateTimeFormatter formatter(Locale locale, Chronology chronology) {
            String string = chronology.getId() + 124 + locale.toString() + 124 + this.dateStyle + this.timeStyle;
            DateTimeFormatter formatter = (DateTimeFormatter)DateTimeFormatterBuilder.LocalizedPrinterParser.FORMATTER_CACHE.get(string);
            if (formatter == null) {
                formatter = new DateTimeFormatterBuilder().appendPattern(DateTimeFormatterBuilder.getLocalizedDateTimePattern(this.dateStyle, this.timeStyle, chronology, locale)).toFormatter(locale);
                DateTimeFormatter dateTimeFormatter = (DateTimeFormatter)DateTimeFormatterBuilder.LocalizedPrinterParser.FORMATTER_CACHE.putIfAbsent(string, formatter);
                if (dateTimeFormatter != null) {
                    formatter = dateTimeFormatter;
                }
            }
            return formatter;
        }
        
        @Override
        public String toString() {
            return "Localized(" + ((this.dateStyle != null) ? this.dateStyle : "") + "," + ((this.timeStyle != null) ? this.timeStyle : "") + ")";
        }
        
        static {
            FORMATTER_CACHE = new ConcurrentHashMap(16, 0.75f, 2);
        }
    }
    
    static final class WeekBasedFieldPrinterParser implements DateTimeFormatterBuilder.DateTimePrinterParser
    {
        private char chr;
        private int count;
        
        WeekBasedFieldPrinterParser(char chr, int count) {
            this.chr = chr;
            this.count = count;
        }
        
        @Override
        public boolean format(DateTimePrintContext dateTimePrintContext, StringBuilder sb) {
            return this.printerParser(dateTimePrintContext.getLocale()).format(dateTimePrintContext, sb);
        }
        
        @Override
        public int parse(DateTimeParseContext dateTimeParseContext, CharSequence charSequence, int n) {
            return this.printerParser(dateTimeParseContext.getLocale()).parse(dateTimeParseContext, charSequence, n);
        }
        
        private DateTimeFormatterBuilder.DateTimePrinterParser printerParser(Locale locale) {
            WeekFields of = WeekFields.of(locale);
            TemporalField temporalField;
            switch (this.chr) {
                case 'Y': {
                    TemporalField weekBasedYear = of.weekBasedYear();
                    if (this.count == 2) {
                        return new DateTimeFormatterBuilder.ReducedPrinterParser(weekBasedYear, 2, 2, 0, DateTimeFormatterBuilder.ReducedPrinterParser.BASE_DATE, 0);
                    }
                    return new DateTimeFormatterBuilder.NumberPrinterParser(weekBasedYear, this.count, 19, (this.count < 4) ? SignStyle.NORMAL : SignStyle.EXCEEDS_PAD, -1);
                }
                case 'c':
                case 'e': {
                    temporalField = of.dayOfWeek();
                    break;
                }
                case 'w': {
                    temporalField = of.weekOfWeekBasedYear();
                    break;
                }
                case 'W': {
                    temporalField = of.weekOfMonth();
                    break;
                }
                default: {
                    throw new IllegalStateException("unreachable");
                }
            }
            return new DateTimeFormatterBuilder.NumberPrinterParser(temporalField, (this.count == 2) ? 2 : 1, 2, SignStyle.NOT_NEGATIVE);
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder(30);
            sb.append("Localized(");
            if (this.chr == 'Y') {
                if (this.count == 1) {
                    sb.append("WeekBasedYear");
                }
                else if (this.count == 2) {
                    sb.append("ReducedValue(WeekBasedYear,2,2,2000-01-01)");
                }
                else {
                    sb.append("WeekBasedYear,").append(this.count).append(",").append(19).append(",").append((this.count < 4) ? SignStyle.NORMAL : SignStyle.EXCEEDS_PAD);
                }
            }
            else {
                switch (this.chr) {
                    case 'c':
                    case 'e': {
                        sb.append("DayOfWeek");
                        break;
                    }
                    case 'w': {
                        sb.append("WeekOfWeekBasedYear");
                        break;
                    }
                    case 'W': {
                        sb.append("WeekOfMonth");
                        break;
                    }
                }
                sb.append(",");
                sb.append(this.count);
            }
            sb.append(")");
            return sb.toString();
        }
    }
    
    interface DateTimePrinterParser
    {
        public abstract boolean format(DateTimePrintContext p0, StringBuilder p1);
        
        public abstract int parse(DateTimeParseContext p0, CharSequence p1, int p2);
    }
}
